@@ -17,47 +17,52 @@
 */
 
 #ifndef STDAFX_H
-#define STDAFX_H
+# define STDAFX_H
 
-extern "C"
-{
-#include "config.h"
+# include "config.h"
 
-#ifdef HAVE_PTHREAD
-# include <pthread.h>
-#endif
-}
+# ifdef HAVE_PTHREAD
+#  include <pthread.h>
+# endif
 
-extern "C"
-{
-#include <stdlib.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <math.h>
-#include "time.h"
-#include <ctype.h>
-#include <stdint.h>
+# include <stdlib.h>
+# include <stdio.h>
+# include <fcntl.h>
+# include <math.h>
+# include "time.h"
+# include <ctype.h>
+# include <stdint.h>
+
+# ifdef WIN32
+#  include <tchar.h>
+#  include <process.h>
+# else
+#  include <sys/types.h>
+#  include <sys/stat.h>
+#  include <limits.h>
+# endif
 
-#ifdef WIN32
-# include <tchar.h>
-# include <process.h>
-#else
-# include <sys/types.h>
-# include <sys/stat.h>
-# include <limits.h>
-#endif
-}
 
 typedef void* HANDLE;
 
 #endif
 
-extern "C"
-{
 #ifdef HAVE_GETTEXT
+
+/*XXX: Fix me.  */
+# undef sprintf
+# undef snprintf
+
 # include <libintl.h>
+
 # define _(X) gettext (X)
+
 #else
 # define _(X) X
 #endif
-}
+
+/*
+  Automatically append an exception to the formatting string.
+  Don't forget to pass as last parameter a pointer to an exception.
+ */
+#define _E(X) _(X " : %e")
