@@ -43,9 +43,9 @@ SecurityDomain::~SecurityDomain ()
 }
 
 /*!
- *Get the stored value for [name].
+  Get the stored value for [name].
  */
-string *SecurityDomain::getValue (string &name)
+const string *SecurityDomain::getValue (string &name)
 {
   return NULL;
 }
