@@ -16,32 +16,29 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "myserver.h"
+#include <myserver.h>
 #include <include/server/server.h>
 #include <include/base/file/files_utility.h>
 #include <include/base/string/stringutils.h>
 #include <include/base/process/process.h>
 #include <include/conf/main/xml_main_configuration.h>
 
-extern "C"
-{
+#include <include/base/exceptions/checked.h>
+
 #ifdef WIN32
 # include <direct.h>
 #endif
 
-# include <argp.h>
+#include <argp.h>
 
 #ifndef WIN32
 # include <string.h>
 # include <unistd.h>
 # include <signal.h>
 #endif
-}
-
-#ifdef HAVE_GETTEXT
-# include "localedir.h"
-#endif
 
+#include <libintl.h>
+#include "localedir.h"
 
 enum
 {
@@ -107,13 +104,12 @@ void registerSignals ()
   sig1.sa_handler = SIG_IGN;
   sig2.sa_handler = Sig_Quit;
   sig3.sa_handler = Sig_Hup;
-  sigaction (SIGPIPE, &sig1, NULL);
-  sigaction (SIGINT, &sig2, NULL);
-  sigaction (SIGTERM, &sig2, NULL);
-  sigaction (SIGHUP, &sig3, NULL);
-
+  checked::sigaction (SIGPIPE, &sig1, NULL);
+  checked::sigaction (SIGINT, &sig2, NULL);
+  checked::sigaction (SIGTERM, &sig2, NULL);
+  checked::sigaction (SIGHUP, &sig3, NULL);
   /* Avoid zombie processes.  */
-  sigaction (SIGCHLD, &sa, (struct sigaction *)NULL);
+  checked::sigaction (SIGCHLD, &sa, (struct sigaction *)NULL);
 #else
   SetConsoleMode (GetStdHandle (STD_INPUT_HANDLE), ENABLE_PROCESSED_INPUT);
   SetConsoleCtrlHandler ((PHANDLER_ROUTINE) SignalHandler, TRUE);
@@ -122,6 +118,17 @@ void registerSignals ()
 
 struct argp_input
 {
+  argp_input ()
+  {
+    version = 0;
+    confFilesLocation = NULL;
+    logFileName = NULL;
+    runas = MYSERVER_RUNAS_CONSOLE;
+    pidFileName = NULL;
+    useForkServer = 0;
+    plugins = NULL;
+  }
+
   /* Print the version for MyServer?  */
   int version;
 
@@ -139,6 +146,9 @@ struct argp_input
 
   /* Specify if the fork server is used.  */
   int useForkServer;
+
+  /* Plugins to load at startup before the server starts.  */
+  const char *plugins;
 };
 
 static char doc[] = "GNU MyServer ";
@@ -146,7 +156,8 @@ static char argsDoc[] = "";
 
 enum
 {
- CONFIG_OPT = UCHAR_MAX + 1
+  CONFIG_OPT = UCHAR_MAX + 1,
+  CONFIG_PLUGINS
 };
 
 /* Use the GNU C argp parser under not windows environments.  */
@@ -166,6 +177,8 @@ static struct argp_option options[] =
      _("Specify if use a fork server")},
     {"cfgdir", CONFIG_OPT, "dir", 0,
      _("Specify an alternative directory where look for configuration files")},
+    {"plugins", CONFIG_PLUGINS, "name1:plugin1,name2:plugin2", 0,
+     _("Specify which plugins must be loaded before the server starts.")},
     {0}
   };
 
@@ -178,12 +191,16 @@ static error_t parseOpt (int key, char *arg, struct argp_state *state)
       in->version = 1;
       break;
 
+    case CONFIG_PLUGINS:
+      in->plugins = arg;
+      break;
+
     case 'r':
       if (arg)
         {
-          if (!strcmpi (arg, "CONSOLE"))
+          if (! strcasecmp (arg, "CONSOLE"))
             in->runas = MYSERVER_RUNAS_CONSOLE;
-          else if (!strcmpi (arg, "SERVICE"))
+          else if (! strcasecmp (arg, "SERVICE"))
             in->runas = MYSERVER_RUNAS_SERVICE;
         }
       else
@@ -219,8 +236,8 @@ static error_t parseOpt (int key, char *arg, struct argp_state *state)
 static struct argp myserverArgp = {options, parseOpt, argsDoc, doc};
 
 /*!
- * Load the external path.
- * Return nonzero on errors.
+  Load the external path.
+  Return nonzero on errors.
  */
 int loadExternalPath (string &externalPath)
 {
@@ -230,7 +247,7 @@ int loadExternalPath (string &externalPath)
 #ifdef WIN32
       externalPath.assign ("plugins");
 #else
-      if (FilesUtility::fileExists ("plugins"))
+      if (FilesUtility::nodeExists ("plugins"))
         externalPath.assign ("plugins");
       else
         {
@@ -251,9 +268,9 @@ int loadExternalPath (string &externalPath)
 
 
 /*!
- * Load the vhost configuration files locations.
- * If DIR is specified, look only in this directory.
- * Return nonzero on errors.
+  Load the vhost configuration files locations.
+  If DIR is specified, look only in this directory.
+  Return nonzero on errors.
  */
 int loadConfFileLocation (string &outFile, string fileName, const char *dir)
 {
@@ -263,7 +280,7 @@ int loadConfFileLocation (string &outFile, string fileName, const char *dir)
 
 #ifdef WIN32
       outFile = fileName;
-      return FilesUtility::fileExists (fileName) ? 0 : -1;
+      return FilesUtility::nodeExists (fileName) ? 0 : -1;
 #else
 
       if (dir)
@@ -273,7 +290,7 @@ int loadConfFileLocation (string &outFile, string fileName, const char *dir)
             outFile += "/";
 
           outFile += fileName;
-          return !FilesUtility::fileExists (outFile);
+          return !FilesUtility::nodeExists (outFile);
         }
 
       /* Look for .xml files in the following order:
@@ -282,18 +299,18 @@ int loadConfFileLocation (string &outFile, string fileName, const char *dir)
          2) ~/.myserver/
          3) /etc/myserver/
       */
-      if (FilesUtility::fileExists (fileName))
+      if (FilesUtility::nodeExists (fileName))
         {
           outFile = fileName;
           return 0;
         }
 
       outFile = "~/.myserver/" + fileName;
-      if (FilesUtility::fileExists (outFile))
+      if (FilesUtility::nodeExists (outFile))
         return 0;
 
       outFile = "/etc/myserver/" + fileName;
-      if (FilesUtility::fileExists (outFile))
+      if (FilesUtility::nodeExists (outFile))
         return 0;
 #endif
     }
@@ -304,8 +321,8 @@ int loadConfFileLocation (string &outFile, string fileName, const char *dir)
 }
 
 /*!
- * Load the configuration files locations.
- * Return nonzero on errors.
+  Load the configuration files locations.
+  Return nonzero on errors.
  */
 int loadConfFilesLocation (string &mainConfigurationFile,
                            string &mimeConfigurationFile,
@@ -328,7 +345,25 @@ int loadConfFilesLocation (string &mainConfigurationFile,
   return 0;
 }
 
-static MainConfiguration *genMainConf (Server *server, const char *arg)
+static void updateWorkingDirectory (const char *firstArgument)
+{
+  string path (firstArgument);
+  size_t index;
+
+  index = path.find_last_of ('/');
+
+#ifdef WIN32
+  index = max (index, path.find_last_of ('\\'));
+#endif
+
+  if (index != string::npos)
+    {
+      string newdir (path, 0, index);
+      setcwd (newdir.c_str ());
+    }
+}
+
+static MainConfiguration *_genMainConf (Server *server, const char *arg)
 {
   XmlMainConfiguration *conf = new XmlMainConfiguration ();
   if (conf->open (arg))
@@ -339,9 +374,11 @@ static MainConfiguration *genMainConf (Server *server, const char *arg)
   return conf;
 }
 
+MainConfiguration* (*genMainConf) (Server *server, const char *arg) = &_genMainConf;
+
 const char *program_name = NULL;
 
-int main  (int argn, char **argv)
+int main (int argn, char **argv)
 {
   program_name = argv[0];
   int runas = MYSERVER_RUNAS_CONSOLE;
@@ -355,62 +392,27 @@ int main  (int argn, char **argv)
 
   registerSignals ();
 
-#if HAVE_GETTEXT
   setlocale (LC_ALL, "");
+  setlocale (LC_TIME, "POSIX");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
-#endif
 
   try
     {
       Server::createInstance ();
     }
+  catch (exception & e)
+    {
+      cerr << e.what () << endl;
+      return 1;
+    }
   catch (...)
     {
-      /* Die if we get exceptions here.  */
-      return (1);
+      return 1;
     };
 
-  u_int pathLen = 0;
-  u_int len = 0;
-  bool differentCwd = false;
-  char *path;
-
-  pathLen = strlen (argv[0]);
-  path = new char[pathLen + 1];
-  if (path == 0)
-    return 1;
-  strncpy (path, argv[0], pathLen);
-
-  for (len = 0; len < pathLen; len++)
-    {
-      if (path[len] == '/' || path[len] == '\\')
-        {
-          differentCwd = true;
-          break;
-        }
-    }
-
-  if (differentCwd)
-    {
-      len = pathLen;
-      while ((path[len] != '\\') && (path[len] != '/'))
-        len--;
-      path[len] = '\0';
-
-      setcwd (path);
-    }
-
-  /* We can free path memory now.  */
-  delete [] path;
-
-  /* Reset the struct.  */
-  input.version = 0;
-  input.confFilesLocation = NULL;
-  input.logFileName = NULL;
-  input.runas = MYSERVER_RUNAS_CONSOLE;
-  input.pidFileName = NULL;
-  input.useForkServer = 0;
+  /* Move to a different working directory, if necessary.  */
+  updateWorkingDirectory (argv[0]);
 
   /* Call the parser.  */
   argp_parse (&myserverArgp, argn, argv, 0, 0, &input);
@@ -419,7 +421,7 @@ int main  (int argn, char **argv)
   if (input.logFileName
       && Server::getInstance ()->setLogLocation (input.logFileName))
     {
-      cout << "Error setting the location for the MyServer's main log" << endl;
+      cerr << "Error setting the location for the MyServer's main log" << endl;
       return 1;
     }
 
@@ -435,26 +437,39 @@ int main  (int argn, char **argv)
            << "http://www.gnu.org/software/myserver" << endl;
       return 0;
     }
+
   if (input.useForkServer)
     {
       FilesUtility::resetTmpPath ();
       Process::getForkServer ()->startForkServer ();
     }
 
+  if (input.plugins)
+  {
+    PluginsManager *pluginsManager = Server::getInstance ()->getPluginsManager ();
+    if (pluginsManager->quickLoad (Server::getInstance (), input.plugins))
+      {
+        cerr << _("Cannot load specified plugins") << endl;
+        return 1;
+      }
+  }
+
   /*
-   * Start here the MyServer execution.
+    Start here the MyServer execution.
    */
   try
     {
-      if (loadConfFilesLocation (mainConf, mimeConf, vhostConf, externPath,
+      const char *confFileDir;
 #ifdef WIN32
-                                 "."
+      confFileDir = ".";
 #else
-                                 input.confFilesLocation
+      confFileDir = input.confFilesLocation;
 #endif
-                                 ))
+
+      if (loadConfFilesLocation (mainConf, mimeConf, vhostConf, externPath,
+                                 confFileDir))
         {
-          cout << _("Cannot find the configuration files, be sure they exist")
+          cerr << _("Cannot find the configuration files, be sure they exist")
                << endl;
           return 1;
         }
@@ -466,19 +481,17 @@ int main  (int argn, char **argv)
           runService ();
 #else
           /*
-           * Run the daemon.
-           * pid is the process ID for the forked process.
-           * Fork the process.
+            Run the daemon.
+            pid is the process ID for the forked process.
+            Fork the process.
            */
           pid = fork ();
 
           /*
-           * An error happened, return with errors.
+            An error happened, return with errors.
            */
           if (pid < 0)
-            {
-              return 1;
-            }
+            return 1;
 
           if (pid)
             {
@@ -488,15 +501,16 @@ int main  (int argn, char **argv)
                 writePidfile ();
               return 0;
             }
+
           /*
-           * Create a SID for the new process.
+            Create a SID for the new process.
            */
           sid = setsid ();
           if (sid < 0)
             return 1;
 #endif
         case MYSERVER_RUNAS_CONSOLE:
-          consoleService (mainConf, mimeConf, vhostConf, externPath, &genMainConf);
+          consoleService (mainConf, mimeConf, vhostConf, externPath, genMainConf);
         }
     }
   catch (...)
@@ -513,7 +527,7 @@ int main  (int argn, char **argv)
 #ifndef WIN32
 
 /*!
- * Write the current PID to the file.
+  Write the current PID to the file.
  */
 int writePidfile (const char* filename)
 {
@@ -532,23 +546,23 @@ int writePidfile (const char* filename)
       file.append ("/var/run/myserver.pid");
     }
 
-  pidfile = open (filename, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+  pidfile = checked::open (filename, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
   if (pidfile == -1)
     return -1;
 
   sprintf (buff,"%i\n", pid);
-  ret = write (pidfile, buff, strlen (buff));
+  ret = checked::write (pidfile, buff, strlen (buff));
   if (ret == -1)
     {
-      close (pidfile);
+      checked::close (pidfile);
       return -1;
     }
-  return close (pidfile);
+  return checked::close (pidfile);
 }
 #endif
 
 /*!
- * Start MyServer in console mode.
+  Start MyServer in console mode.
  */
 void consoleService (string &mainConf, string &mimeConf, string &vhostConf,
                      string &externPath, MainConfiguration* (*genMainConf)
@@ -559,14 +573,14 @@ void consoleService (string &mainConf, string &mimeConf, string &vhostConf,
 }
 
 /*!
- * These functions are available only on the windows platform.
+  These functions are available only on the windows platform.
  */
 #ifdef WIN32
 SERVICE_STATUS          MyServiceStatus;
 SERVICE_STATUS_HANDLE   MyServiceStatusHandle;
 
 /*!
- * Entry-point for the NT service.
+  Entry-point for the NT service.
  */
 void  __stdcall myServerMainNT (u_long, LPTSTR*)
 {
@@ -606,7 +620,7 @@ void  __stdcall myServerMainNT (u_long, LPTSTR*)
 
       loadConfFilesLocation (mainConf, mimeConf, vhostConf, externPath, NULL);
       Server::getInstance ()->start (mainConf, mimeConf, vhostConf, externPath,
-                                     &genMainConf);
+                                     genMainConf);
 
       MyServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
       SetServiceStatus (MyServiceStatusHandle, &MyServiceStatus);
@@ -620,7 +634,7 @@ void  __stdcall myServerMainNT (u_long, LPTSTR*)
 }
 
 /*!
- * Manage the NT service.
+  Manage the NT service.
  */
 void __stdcall myServerCtrlHandler (u_long fdwControl)
 {
@@ -648,7 +662,7 @@ void __stdcall myServerCtrlHandler (u_long fdwControl)
 #endif
 
 /*!
- * Run MyServer service.
+  Run MyServer service.
  */
 void runService ()
 {
@@ -673,7 +687,7 @@ void runService ()
 }
 
 /*!
- * Register the service.
+  Register the service.
  */
 void registerService ()
 {
@@ -702,7 +716,7 @@ void registerService ()
 }
 
 /*
- *Unregister the OS service.
+  Unregister the OS service.
  */
 void removeService ()
 {
@@ -727,7 +741,7 @@ void removeService ()
 }
 
 /*!
- * Start the service.
+  Start the service.
  */
 void runAsService ()
 {
