@@ -31,6 +31,8 @@
 #include <include/base/string/stringutils.h>
 #include <include/base/mem_buff/mem_buff.h>
 
+#include <include/base/exceptions/checked.h>
+
 #include <include/conf/security/auth_domain.h>
 
 #ifndef WIN32
@@ -45,6 +47,19 @@ static DEFINE_THREAD (SendImageFile, pParam);
 static DEFINE_THREAD (ReceiveAsciiFile, pParam);
 static DEFINE_THREAD (ReceiveImageFile, pParam);
 
+
+/* FIXME: move somewhere else.  */
+static bool
+areSymlinkAllowed (SecurityToken *st)
+{
+  const char *perm = st->getData ("symlinks.follow",
+                                  MYSERVER_VHOST_CONF
+                                  | MYSERVER_SERVER_CONF,
+                                  "NO");
+  return strcasecmp (perm, "YES") == 0;
+}
+
+
 void setFtpHost (FtpHost & out, const FtpHost & in)
 {
   out.h1 = in.h1;
@@ -58,7 +73,7 @@ void setFtpHost (FtpHost & out, const FtpHost & in)
 void setFtpHost (FtpHost & out, const char *szIn)
 {
   std::stringstream ss;
-  char *szLocalIn = strdup (szIn);
+  char *szLocalIn = checked::strdup (szIn);
   char *tok = strtok (szLocalIn, ",.");
   while (tok != NULL)
     {
@@ -109,13 +124,6 @@ FtpuserData::FtpuserData ()
 
 bool FtpuserData::allowdelete (bool wait)
 {
-  if (wait)
-    {
-      /* Wait for data connection to finish.  */
-      m_DataConnBusy.lock ();
-      m_DataConnBusy.unlock ();
-
-    }
   if (m_pDataConnection != NULL)
     return !m_pDataConnection->isScheduled ();
   else
@@ -338,7 +346,7 @@ int Ftp::controlConnection (ConnectionPtr pConnection, char *request,
     {
       pFtpuserData->m_nFtpstate = FtpuserData::UNAVAILABLE;
       ftpReply (421);
-      return 0;
+      return ClientsThread::DELETE_CONNECTION;
     }
 
   m_nLocalControlport = pConnection->getLocalPort ();
@@ -355,7 +363,16 @@ int Ftp::controlConnection (ConnectionPtr pConnection, char *request,
   td.pProtocolInterpreter = this;
   td.m_nParseLength = 0;
 
-  return parseControlConnection ();
+  try
+    {
+      return parseControlConnection ();
+    }
+  catch (exception & e)
+    {
+      return ClientsThread::DELETE_CONNECTION;
+    }
+
+  return ClientsThread::DELETE_CONNECTION;
 }
 
 
@@ -376,23 +393,23 @@ int Ftp::loadProtocolstatic ()
 
   const char *pData = server->getData ("ftp.allow_anonymous");
   if (pData != NULL)
-    m_ballowAnonymous = strcmpi ("Yes", pData) == 0 ? true : false;
+    m_ballowAnonymous = strcasecmp ("Yes", pData) == 0 ? true : false;
 
   pData = server->getData ("ftp.anonymous_need_pass");
   if (pData != NULL)
-    m_bAnonymousNeedPass = strcmpi ("Yes", pData) == 0 ? true : false;
+    m_bAnonymousNeedPass = strcasecmp ("Yes", pData) == 0 ? true : false;
 
   pData = server->getData ("ftp.allow_asynchronous_cmds");
   if (pData != NULL)
-    m_ballowAsynchronousCmds = strcmpi ("Yes", pData) == 0 ? true : false;
+    m_ballowAsynchronousCmds = strcasecmp ("Yes", pData) == 0 ? true : false;
 
   pData = server->getData ("ftp.allow_pipelining");
   if (pData != NULL)
-    m_bEnablePipelining = strcmpi ("Yes", pData) == 0 ? true : false;
+    m_bEnablePipelining = strcasecmp ("Yes", pData) == 0 ? true : false;
 
   pData = server->getData ("ftp.allow_store");
   if (pData != NULL)
-    m_bEnablestoreCmds = strcmpi ("Yes", pData) == 0 ? true : false;
+    m_bEnablestoreCmds = strcasecmp ("Yes", pData) == 0 ? true : false;
 
   return 1;
 }
@@ -449,7 +466,7 @@ Ftp::user (const std::string & sParam)
     static_cast < FtpuserData * >(td.pConnection->protocolBuffer);
 
   if (!m_ballowAnonymous
-      && strcmpi (pFtpuserData->m_suserName.c_str (), "anonymous") == 0)
+      && strcasecmp (pFtpuserData->m_suserName.c_str (), "anonymous") == 0)
     {
       ftpReply (530);
       return;
@@ -465,7 +482,7 @@ void Ftp::password (const std::string & sParam)
     static_cast < FtpuserData * >(td.pConnection->protocolBuffer);
 
   if (!m_ballowAnonymous
-      && strcmpi (pFtpuserData->m_suserName.c_str (), "anonymous") == 0)
+      && strcasecmp (pFtpuserData->m_suserName.c_str (), "anonymous") == 0)
     {
       ftpReply (530);
       return;
@@ -573,7 +590,7 @@ void Ftp::retrstor (bool bretr, bool bappend, const std::string & sPath)
                                        MYSERVER_VHOST_CONF |
                                        MYSERVER_SERVER_CONF,
                                        ".security.xml");
-  if (!strcmpi (sLocalFileName.c_str (), secName))
+  if (! strcasecmp (sLocalFileName.c_str (), secName))
     {
       ftpReply (550);
       return;
@@ -610,6 +627,7 @@ void Ftp::retrstor (bool bretr, bool bappend, const std::string & sPath)
   pData->m_bappend = bappend || pFtpuserData->m_nrestartOffset > 0;
   pData->m_sFilePath = sLocalPath;
   pData->m_pFtp = this;
+  pData->st = &td.st;
 
   pFtpuserData->m_sCurrentFileName = "";
   pFtpuserData->m_nFileSize = 0;
@@ -631,6 +649,7 @@ void Ftp::retrstor (bool bretr, bool bappend, const std::string & sPath)
 static
 DEFINE_THREAD (SendAsciiFile, pParam)
 {
+  File *file = NULL;
   DataConnectionWorkerThreadData *pWt =
     reinterpret_cast < DataConnectionWorkerThreadData * >(pParam);
   if (pWt == NULL)
@@ -668,27 +687,11 @@ DEFINE_THREAD (SendAsciiFile, pParam)
     }
 
   pFtpuserData->m_DataConnBusy.lock ();
-
-  if (pWt->m_pFtp == NULL)
+  try
     {
-      pFtpuserData->closeDataConnection ();
-      pFtpuserData->m_DataConnBusy.unlock ();
-      delete pWt;
-#ifdef WIN32
-      return 0;
-#elif HAVE_PTHREAD
-      return (void *) 0;
-#endif
-    }
 
-  if (pFtpuserData->m_nFtpstate == FtpuserData::DATA_CONNECTION_UP)
-    ftpReply (pConnection, 125);
-  else
-    {
-      ftpReply (pConnection, 150);
-      if (pWt->m_pFtp->OpenDataConnection () == 0)
+      if (pWt->m_pFtp == NULL)
         {
-          ftpReply (pConnection, 425);
           pFtpuserData->closeDataConnection ();
           pFtpuserData->m_DataConnBusy.unlock ();
           delete pWt;
@@ -698,27 +701,45 @@ DEFINE_THREAD (SendAsciiFile, pParam)
           return (void *) 0;
 #endif
         }
-    }
 
-  if (pFtpuserData->m_pDataConnection == NULL ||
-      pFtpuserData->m_pDataConnection->socket == NULL)
-    {
-      ftpReply (pConnection, 451);
-      pFtpuserData->closeDataConnection ();
-      pFtpuserData->m_DataConnBusy.unlock ();
-      delete pWt;
+      if (pFtpuserData->m_nFtpstate == FtpuserData::DATA_CONNECTION_UP)
+        ftpReply (pConnection, 125);
+      else
+        {
+          ftpReply (pConnection, 150);
+          if (pWt->m_pFtp->OpenDataConnection () == 0)
+            {
+              ftpReply (pConnection, 425);
+              pFtpuserData->closeDataConnection ();
+              pFtpuserData->m_DataConnBusy.unlock ();
+              delete pWt;
 #ifdef WIN32
-      return 0;
+              return 0;
 #elif HAVE_PTHREAD
-      return (void *) 0;
+              return (void *) 0;
 #endif
-    }
-  File *file = NULL;
-  try
-    {
+            }
+        }
+
+      if (pFtpuserData->m_pDataConnection == NULL ||
+          pFtpuserData->m_pDataConnection->socket == NULL)
+        {
+          ftpReply (pConnection, 451);
+          pFtpuserData->closeDataConnection ();
+          pFtpuserData->m_DataConnBusy.unlock ();
+          delete pWt;
+#ifdef WIN32
+          return 0;
+#elif HAVE_PTHREAD
+          return (void *) 0;
+#endif
+        }
+
+      int symFlags = areSymlinkAllowed (pWt->st) ? 0
+                                                 : File::NO_FOLLOW_SYMLINK;
       file =
         Server::getInstance ()->getCachedFiles ()->open (pWt->m_sFilePath.
-                                                         c_str ());
+                                                         c_str (), symFlags);
       if (file == NULL)
         {
           ftpReply (pConnection, 451);
@@ -731,7 +752,7 @@ DEFINE_THREAD (SendAsciiFile, pParam)
           return (void *) 0;
 #endif
         }
-      u_long filesize = file->getFileSize ();
+      size_t filesize = file->getFileSize ();
 
       /* don't implement restart for ASCII.  */
       if (pFtpuserData->m_nrestartOffset > 0)
@@ -739,7 +760,7 @@ DEFINE_THREAD (SendAsciiFile, pParam)
       pFtpuserData->m_sCurrentFileName = pWt->m_sFilePath;
       pFtpuserData->m_nFileSize = filesize;
 
-      u_long nbr, nBufferSize = 0;
+      size_t nbr, nBufferSize = 0;
       char *pLine = NULL;
       int nLineLength = 0;
       std::string sLine;
@@ -839,21 +860,23 @@ DEFINE_THREAD (SendAsciiFile, pParam)
         }
       file->close ();
       delete file;
+
+      pFtpuserData->m_sCurrentFileName = "";
+      pFtpuserData->m_nFileSize = 0;
+      pFtpuserData->m_nBytesSent = 0;
+      pFtpuserData->m_nrestartOffset = 0;
+      ftpReply (pConnection, 226);
+      pFtpuserData->closeDataConnection ();
+      pFtpuserData->m_DataConnBusy.unlock ();
     }
-  catch (bad_alloc & ba)
+  catch (exception & e)
     {
       if (file != NULL)
         file->close ();
       delete file;
+      pFtpuserData->m_DataConnBusy.unlock ();
     }
 
-  pFtpuserData->m_sCurrentFileName = "";
-  pFtpuserData->m_nFileSize = 0;
-  pFtpuserData->m_nBytesSent = 0;
-  pFtpuserData->m_nrestartOffset = 0;
-  ftpReply (pConnection, 226);
-  pFtpuserData->closeDataConnection ();
-  pFtpuserData->m_DataConnBusy.unlock ();
   delete pWt;
 #ifdef WIN32
   return 1;
@@ -904,27 +927,11 @@ DEFINE_THREAD (SendImageFile, pParam)
     }
 
   pFtpuserData->m_DataConnBusy.lock ();
-
-  if (pWt->m_pFtp == NULL)
+  File *file = NULL;
+  try
     {
-      pFtpuserData->closeDataConnection ();
-      pFtpuserData->m_DataConnBusy.unlock ();
-      delete pWt;
-#ifdef WIN32
-      return 0;
-#elif HAVE_PTHREAD
-      return (void *) 0;
-#endif
-    }
-
-  if (pFtpuserData->m_nFtpstate == FtpuserData::DATA_CONNECTION_UP)
-    ftpReply (pConnection, 125);
-  else
-    {
-      ftpReply (pConnection, 150);
-      if (pWt->m_pFtp->OpenDataConnection () == 0)
+      if (pWt->m_pFtp == NULL)
         {
-          ftpReply (pConnection, 425);
           pFtpuserData->closeDataConnection ();
           pFtpuserData->m_DataConnBusy.unlock ();
           delete pWt;
@@ -934,28 +941,45 @@ DEFINE_THREAD (SendImageFile, pParam)
           return (void *) 0;
 #endif
         }
-    }
 
-  if (pFtpuserData->m_pDataConnection == NULL ||
-      pFtpuserData->m_pDataConnection->socket == NULL)
-    {
-      ftpReply (pConnection, 451);
-      pFtpuserData->closeDataConnection ();
-      pFtpuserData->m_DataConnBusy.unlock ();
-      delete pWt;
+      if (pFtpuserData->m_nFtpstate == FtpuserData::DATA_CONNECTION_UP)
+        ftpReply (pConnection, 125);
+      else
+        {
+          ftpReply (pConnection, 150);
+          if (pWt->m_pFtp->OpenDataConnection () == 0)
+            {
+              ftpReply (pConnection, 425);
+              pFtpuserData->closeDataConnection ();
+              pFtpuserData->m_DataConnBusy.unlock ();
+              delete pWt;
 #ifdef WIN32
-      return 0;
+              return 0;
 #elif HAVE_PTHREAD
-      return (void *) 0;
+              return (void *) 0;
 #endif
-    }
+            }
+        }
 
-  File *file = NULL;
-  try
-    {
+      if (pFtpuserData->m_pDataConnection == NULL ||
+          pFtpuserData->m_pDataConnection->socket == NULL)
+        {
+          ftpReply (pConnection, 451);
+          pFtpuserData->closeDataConnection ();
+          pFtpuserData->m_DataConnBusy.unlock ();
+          delete pWt;
+#ifdef WIN32
+          return 0;
+#elif HAVE_PTHREAD
+          return (void *) 0;
+#endif
+        }
+
+      int symFlags = areSymlinkAllowed (pWt->st) ? 0
+                                                 : File::NO_FOLLOW_SYMLINK;
       file =
         Server::getInstance ()->getCachedFiles ()->open (pWt->m_sFilePath.
-                                                         c_str ());
+                                                         c_str (), symFlags);
       if (file == NULL)
         {
           ftpReply (pConnection, 451);
@@ -968,8 +992,8 @@ DEFINE_THREAD (SendImageFile, pParam)
           return (void *) 0;
 #endif
         }
-      u_long filesize = file->getFileSize ();
-      u_long nbr, nBufferSize = 0;
+      size_t filesize = file->getFileSize ();
+      size_t nbr, nBufferSize = 0;
       if (pWt->m_bappend && pFtpuserData->m_nrestartOffset < filesize)
         {
           file->seek (pFtpuserData->m_nrestartOffset);
@@ -1026,11 +1050,12 @@ DEFINE_THREAD (SendImageFile, pParam)
       file->close ();
       delete file;
     }
-  catch (bad_alloc & ba)
+  catch (exception & e)
     {
       if (file != NULL)
         file->close ();
       delete file;
+      pFtpuserData->m_DataConnBusy.unlock ();
     }
 
   pFtpuserData->m_sCurrentFileName = "";
@@ -1040,6 +1065,7 @@ DEFINE_THREAD (SendImageFile, pParam)
   ftpReply (pConnection, 226);
   pFtpuserData->closeDataConnection ();
   pFtpuserData->m_DataConnBusy.unlock ();
+
   delete pWt;
 #ifdef WIN32
   return 1;
@@ -1142,7 +1168,9 @@ DEFINE_THREAD (ReceiveAsciiFile, pParam)
       if (pWt->m_bappend)
         flags = File::APPEND | File::WRITE;
       else
-        flags = File::FILE_CREATE_ALWAYS | File::WRITE;
+        flags = File::FILE_OPEN_ALWAYS | File::WRITE;
+      flags |= areSymlinkAllowed (pWt->st) ? 0 : File::NO_FOLLOW_SYMLINK;
+
       if (file.openFile (pWt->m_sFilePath.c_str (), flags))
         {
           ftpReply (pConnection, 451);
@@ -1162,10 +1190,10 @@ DEFINE_THREAD (ReceiveAsciiFile, pParam)
       char *pLine = NULL;
       int nLineLength = 0;
       std::string sLine;
-      u_long nbr;
+      size_t nbr;
       Socket *socket = pFtpuserData->m_pDataConnection->socket;
       while (!socket->read (buffer.getBuffer (),
-                           (u_long) buffer.getRealLength () - 1, &nbr)
+                           (size_t) buffer.getRealLength () - 1, &nbr)
              && nbr != 0)
         {
           memset (auxiliaryBuffer.getBuffer (), 0,
@@ -1211,7 +1239,7 @@ DEFINE_THREAD (ReceiveAsciiFile, pParam)
                 }
             }
           file.write (auxiliaryBuffer.getBuffer (),
-                      (u_long) auxiliaryBuffer.getLength (), &nbr);
+                      (size_t) auxiliaryBuffer.getLength (), &nbr);
 
           if (pFtpuserData->m_bBreakDataConnection)
             {
@@ -1230,7 +1258,7 @@ DEFINE_THREAD (ReceiveAsciiFile, pParam)
         }
       file.close ();
     }
-  catch (bad_alloc & ba)
+  catch (exception & e)
     {
       file.close ();
     }
@@ -1340,7 +1368,9 @@ DEFINE_THREAD (ReceiveImageFile, pParam)
       if (pWt->m_bappend)
         flags = File::APPEND | File::WRITE;
       else
-        flags = File::FILE_CREATE_ALWAYS | File::WRITE;
+        flags = File::FILE_OPEN_ALWAYS | File::WRITE;
+      flags |= areSymlinkAllowed (pWt->st) ? 0 : File::NO_FOLLOW_SYMLINK;
+
       if (file.openFile (pWt->m_sFilePath.c_str (), flags))
         {
           ftpReply (pConnection, 451);
@@ -1353,7 +1383,7 @@ DEFINE_THREAD (ReceiveImageFile, pParam)
           return (void *) 0;
 #endif
         }
-      u_long nbr;
+      size_t nbr;
       MemBuf buffer;
       buffer.setRealLength (1024);
       memset (buffer.getBuffer (), 0, buffer.getRealLength ());
@@ -1379,7 +1409,7 @@ DEFINE_THREAD (ReceiveImageFile, pParam)
         }
       file.close ();
     }
-  catch (bad_alloc & ba)
+  catch (exception & e)
     {
       file.close ();
     }
@@ -1412,10 +1442,10 @@ bool Ftp::userLoggedIn ()
 }
 
 /*!
- *Converts from relative client's path to local path (out path may not exist).
- *\param sPathIn client's relative path
- *\param sOutPath local path
- *\return Return true if path exist, file is a normal one and is into the ftp's root folder
+  Converts from relative client's path to local path (out path may not exist).
+  \param sPathIn client's relative path
+  \param sOutPath local path
+  \return Return true if path exist, file is a normal one and is into the ftp's root folder
  */
 bool
 Ftp::buildLocalPath (const std::string & sPathIn, std::string & sOutPath)
@@ -1452,10 +1482,10 @@ Ftp::buildLocalPath (const std::string & sPathIn, std::string & sOutPath)
 }
 
 /*!
- *Converts from relative client's path to local path and checks if the path is available.
- *\param sPath client's relative path
- *\param sOutPath local path
- *\return Return true if path exist, file is a normal one and is into the ftp's root folder
+  Converts from relative client's path to local path and checks if the path is available.
+  \param sPath client's relative path
+  \param sOutPath local path
+  \return Return true if path exist, file is a normal one and is into the ftp's root folder
  */
 bool Ftp::getLocalPath (const std::string & sPath, std::string & sOutPath)
 {
@@ -1463,7 +1493,7 @@ bool Ftp::getLocalPath (const std::string & sPath, std::string & sOutPath)
     return false;
 
   if (sOutPath.empty () ||
-      !FilesUtility::fileExists (sOutPath) ||
+      !FilesUtility::nodeExists (sOutPath) ||
       FilesUtility::isLink (sOutPath.c_str ()))
     {
       ftpReply (550);
@@ -1528,38 +1558,41 @@ Ftp::openDataPassive ()
     static_cast < FtpuserData * >(td.pConnection->protocolBuffer);
 
   Socket *pSocket = new Socket ();
-  pSocket->socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
-  if (pSocket->getHandle () < 0)
-    return 0;
 
-  int nReuseAddr = 1;
-  MYSERVER_SOCKADDR_STORAGE storage = { 0 };
-  ((sockaddr_in *) (&storage))->sin_family = AF_INET;
-  char szIpAddr[16];
-  memset (szIpAddr, 0, 16);
-  getIpAddr (pFtpuserData->m_cdh, szIpAddr, 16);
+  try
+    {
+      pSocket->socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
+      MYSERVER_SOCKADDR_STORAGE storage = { 0 };
+      ((sockaddr_in *) (&storage))->sin_family = AF_INET;
+      char szIpAddr[16];
+      memset (szIpAddr, 0, 16);
+      getIpAddr (pFtpuserData->m_cdh, szIpAddr, 16);
 #ifdef WIN32
-  ((sockaddr_in *) (&storage))->sin_addr.s_addr = inet_addr (szIpAddr);
+      ((sockaddr_in *) (&storage))->sin_addr.s_addr = inet_addr (szIpAddr);
 #else
-  inet_aton (szIpAddr, &((sockaddr_in *) (&storage))->sin_addr);
+      inet_aton (szIpAddr, &((sockaddr_in *) (&storage))->sin_addr);
 #endif
-  ((sockaddr_in *) (&storage))->sin_port =
-    htons (getPortNo (pFtpuserData->m_cdh));
-  if (pSocket->setsockopt (SOL_SOCKET, SO_REUSEADDR, (const char *) &nReuseAddr,
-                           sizeof (nReuseAddr)) < 0)
-    return 0;
+      ((sockaddr_in *) (&storage))->sin_port =
+        htons (getPortNo (pFtpuserData->m_cdh));
+      pSocket->reuseAddress (true);
 
-  if (pSocket->bind (&storage, sizeof (sockaddr_in)) != 0
-      || pSocket->listen (SOMAXCONN) != 0)
-    return 0;
+      pSocket->bind (&storage, sizeof (sockaddr_in));
+      pSocket->listen (SOMAXCONN);
+
+      pFtpuserData->m_pDataConnection->setPort (getPortNo (pFtpuserData->m_cdh));
+      pFtpuserData->m_pDataConnection->setLocalPort (pFtpuserData->m_nLocalDataport);
+      pFtpuserData->m_pDataConnection->setIpAddr (td.pConnection->getIpAddr ());
+      pFtpuserData->m_pDataConnection->setLocalIpAddr (td.pConnection->getLocalIpAddr ());
+      pFtpuserData->m_pDataConnection->host = td.pConnection->host;
+      pFtpuserData->m_pDataConnection->socket = pSocket;
+      pFtpuserData->m_pDataConnection->setScheduled (1);
+    }
+  catch (...)
+    {
+      delete pSocket;
+      return 0;
+    }
 
-  pFtpuserData->m_pDataConnection->setPort (getPortNo (pFtpuserData->m_cdh));
-  pFtpuserData->m_pDataConnection->setLocalPort (pFtpuserData->m_nLocalDataport);
-  pFtpuserData->m_pDataConnection->setIpAddr (td.pConnection->getIpAddr ());
-  pFtpuserData->m_pDataConnection->setLocalIpAddr (td.pConnection->getLocalIpAddr ());
-  pFtpuserData->m_pDataConnection->host = td.pConnection->host;
-  pFtpuserData->m_pDataConnection->socket = pSocket;
-  pFtpuserData->m_pDataConnection->setScheduled (1);
   return 1;
 }
 
@@ -1717,7 +1750,7 @@ Ftp::list (const std::string & sParam /*= ""*/ )
                                            ".security.xml");
       do
         {
-          if (fd.name[0] == '.' || !strcmpi (fd.name.c_str (), secName))
+          if (fd.name[0] == '.' || ! strcasecmp (fd.name.c_str (), secName))
             continue;
 
           perm[10] = '\0';
@@ -1755,7 +1788,7 @@ Ftp::list (const std::string & sParam /*= ""*/ )
           dateFtpFormat[2] = datePtr[9 + offset];
 
           /* If the file was modified in the last 6 months
-           * show the hour instead of the year.  */
+            show the hour instead of the year.  */
           if (now - fd.time_write < 60 * 60 * 183 && now > fd.time_write)
             {
               dateFtpFormat[7] = datePtr[16 + offset];
@@ -1841,7 +1874,7 @@ Ftp::list (const std::string & sParam /*= ""*/ )
           dateFtpFormat[2] = datePtr[9 + offset];
 
           /* If the file was modified in the last 6 months
-           * show the hour instead of the year.  */
+            show the hour instead of the year.  */
           if (now - fd.time_write < 60 * 60 * 183 && now > fd.time_write)
             {
               dateFtpFormat[7] = datePtr[16 + offset];
@@ -1934,7 +1967,7 @@ Ftp::nlst (const std::string & sParam /* = "" */ )
                                        ".security.xml");
   do
     {
-      if (fd.name[0] == '.' || !strcmpi (fd.name.c_str (), secName))
+      if (fd.name[0] == '.' || ! strcasecmp (fd.name.c_str (), secName))
         continue;
 
       if (!sParam.empty ())
@@ -1955,15 +1988,15 @@ Ftp::nlst (const std::string & sParam /* = "" */ )
 }
 
 /*!
- *Handle telnet commands:
- * DOx      -> WONT
- * DONTx    -> ignore
- * WILLx    -> DONT
- * WONTx    -> ignore
- * IACx     -> ignore
- * IACIAC   -> IAC
- *\param In client's requests
- *\param Out client's requests without telnet codes
+  Handle telnet commands:
+  DOx      -> WONT
+  DONTx    -> ignore
+  WILLx    -> DONT
+  WONTx    -> ignore
+  IACx     -> ignore
+  IACIAC   -> IAC
+  \param In client's requests
+  \param Out client's requests without telnet codes
  */
 void
 Ftp::escapeTelnet (MemBuf & In, MemBuf & Out)
@@ -2013,9 +2046,9 @@ Ftp::escapeTelnet (MemBuf & In, MemBuf & Out)
 }
 
 /*!
- *Let only first cmd to be handled.
- *\param In client's requests
- *\param Out client's requests without telnet codes
+  Let only first cmd to be handled.
+  \param In client's requests
+  \param Out client's requests without telnet codes
  */
 void Ftp::removePipelinedCmds (MemBuf & In, MemBuf & Out)
 {
@@ -2194,7 +2227,7 @@ void Ftp::stou (const std::string & sPath)
         return;
       nCount++;
     }
-  while (FilesUtility::fileExists (sOutPath));
+  while (FilesUtility::nodeExists (sOutPath));
   retrstor (false, false, sOutPath);
 }
 
@@ -2217,7 +2250,7 @@ void Ftp::dele (const std::string & sPath)
                                        MYSERVER_VHOST_CONF |
                                        MYSERVER_SERVER_CONF,
                                        ".security.xml");
-  if (!strcmpi (sLocalFileName.c_str (), secName))
+  if (! strcasecmp (sLocalFileName.c_str (), secName))
     {
       ftpReply (550);
       closeDataConnection ();
@@ -2226,7 +2259,7 @@ void Ftp::dele (const std::string & sPath)
   FtpuserData *pFtpuserData =
     static_cast < FtpuserData * >(td.pConnection->protocolBuffer);
 
-  if (strcmpi (pFtpuserData->m_suserName.c_str (), "anonymous") == 0)
+  if (strcasecmp (pFtpuserData->m_suserName.c_str (), "anonymous") == 0)
     {
       if (checkRights ("Guest", "", sLocalFileName, MYSERVER_PERMISSION_WRITE)
           == 0)
@@ -2348,7 +2381,7 @@ void Ftp::rnfr (const std::string & sPath)
                                        ".security.xml");
 
   /* The security file doesn't exist in any case.  */
-  if (!strcmpi (sLocalFileName.c_str (), secName))
+  if (! strcasecmp (sLocalFileName.c_str (), secName))
     {
       ftpReply (550);
       return;
@@ -2393,7 +2426,7 @@ void Ftp::Rnto (const std::string & sPath)
                                        ".security.xml");
 
   /* The security file doesn't exist in any case.  */
-  if (!strcmpi (sLocalFileName.c_str (), secName))
+  if (! strcasecmp (sLocalFileName.c_str (), secName))
     {
       ftpReply (550);
       return;
@@ -2418,7 +2451,7 @@ int Ftp::checkRights (const std::string & suser, const std::string & sPass,
 
   string user;
   string password;
-  if (strcmpi (suser.c_str (), "anonymous") == 0)
+  if (strcasecmp (suser.c_str (), "anonymous") == 0)
     {
       user.assign ("Guest");
       password.assign ("");
@@ -2469,7 +2502,7 @@ void Ftp::size (const std::string & sPath)
                                        MYSERVER_SERVER_CONF,
                                        ".security.xml");
 
-  if (!strcmpi (sLocalFileName.c_str (), secName))
+  if (! strcasecmp (sLocalFileName.c_str (), secName))
     {
       ftpReply (550);
       closeDataConnection ();
@@ -2489,14 +2522,16 @@ void Ftp::size (const std::string & sPath)
     }
 
   File f;
-  if (f.openFile (sLocalPath.c_str (), File::OPEN_IF_EXISTS | File::READ))
+  int flags = File::OPEN_IF_EXISTS | File::READ;
+  flags |= areSymlinkAllowed (&td.st) ? 0 : File::NO_FOLLOW_SYMLINK;
+  if (f.openFile (sLocalPath.c_str (), flags))
     {
       ftpReply (550);
       return;
     }
 
   char size[12];
-  sprintf (size, "%lu", static_cast <u_long> (f.getFileSize ()));
+  sprintf (size, "%llu", static_cast <unsigned long long> (f.getFileSize ()));
   f.close ();
 
   ftpReply (213, size);
